//-----------------------------------------------------------------------------
// Fnnn_TargetBL_DevSpecific.c
//-----------------------------------------------------------------------------
// Copyright (C) 2010 Silicon Laboratories, Inc.
// http://www.silabs.com
//
// Program Description:
//
// [*** TEMPLATE INFORMATION ***]
// Filename: Fnnn_TargetBL_DevSpecific.c
// nnn => Fill-in with the MCU family part number.
// Examples: F50x_TargetBL_DevSpecific.c, F34x_TargetBL_DevSpecific.c
//
//
// Target:         C8051Fxxx (Any Silicon Labs Flash MCU)
// Tool chain:     Generic
// Command Line:   None
//
//
// Release 1.0 / 01SEP2010 (PKC)
//    -Initial Revision
//
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
// Includes
//-----------------------------------------------------------------------------
#include <si_toolchain.h>
#include "Fxxx_SFR_Defs_Wrapper.h"     // This header file will include the
                                       // real MCU register definition file

#include "Fxxx_Master_Config.h"
#include "Fxxx_Master_Interface.h"
#include "Fxxx_Target_Interface.h"
#include "Fxxx_DataSource_Interface.h"
#include "Fxxx_Master_Validation.h"

//-----------------------------------------------------------------------------
// Global CONSTANTS
//-----------------------------------------------------------------------------

// ------------------------------------------
// *** Device-Specific *** 
// Change this as necessary based on MCU
// ------------------------------------------
SI_SBIT(P1_4_SWITCH, SFR_P1, 4);
SI_SBIT(P1_3_LED, SFR_P1, 3);

#define LIN_TX_PUSH_PULL 0 // 0=Open-drain; 1=Push-pull output

//-----------------------------------------------------------------------------
// Global Variables
//-----------------------------------------------------------------------------


//-----------------------------------------------------------------------------
// Function Prototypes (Local)
//-----------------------------------------------------------------------------
void OSCILLATOR_Init (void);
void PORT_Init (void);


//=============================================================================
// Function Definitions
//=============================================================================

//-----------------------------------------------------------------------------
// Disable_Watchdog
//-----------------------------------------------------------------------------
//
// Return Value:  None
// Parameters:    None
//
//
//
//-----------------------------------------------------------------------------
void Disable_Watchdog (void)
{
   // *** Device-Specific *** Change this as necessary based on MCU.

   uint8_t SFRPAGE_save = SFRPAGE;
   SFRPAGE = ACTIVE_PAGE;

   PCA0MD &= ~0x40;

   SFRPAGE = SFRPAGE_save;
}

//-----------------------------------------------------------------------------
// Init_Device
//-----------------------------------------------------------------------------
//
// Return Value:  None
// Parameters:    None
//
//
//
//-----------------------------------------------------------------------------
void Init_Device (void)
{
   OSCILLATOR_Init ();
   PORT_Init ();

   SRC_Comm_Init ();

   Comm_Init ();

   // *** Add calls to any other init functions here as necessary ***
}

//-----------------------------------------------------------------------------
// OSCILLATOR_Init
//-----------------------------------------------------------------------------
//
// Return Value : None
// Parameters   : None
//
// This routine initializes the system clock to use the internal oscillator
// at its maximum frequency (24 MHz); Also enables the VDD monitor set to its
// high threshold and selected as a reset source.
//
//-----------------------------------------------------------------------------
void OSCILLATOR_Init (void)
{
   // *** Device-Specific *** 
   // TODO: Add this based on MCU.

   uint8_t SFRPAGE_save = SFRPAGE;
   SFRPAGE = CONFIG_PAGE;

   OSCICN |= 0xC7;                     // Configure internal oscillator for
                                       // its maximum frequency

   SFRPAGE = ACTIVE_PAGE;

   VDM0CN = 0xA0;                      // Enable VDD monitor and high threshold

   RSTSRC = 0x02;                      // Enable VDD monitor as a reset source

   SFRPAGE = SFRPAGE_save;
}

//-----------------------------------------------------------------------------
// PORT_Init
//-----------------------------------------------------------------------------
//
// Return Value : None
// Parameters   : None
//
// Configure the Crossbar and GPIO ports.
//
// P0.4   digital   push-pull    UART0 TX
// P0.5   digital   open-drain   UART0 RX
//
// P1.0   digital  open-drain    LIN0 TX (see condition below)
// P1.1   digital  open-drain    LIN0 RX
//
// P1.3   digital  push-pull     LED
// P1.4   digital  open-drain    SW

//-----------------------------------------------------------------------------
void PORT_Init (void)
{
   uint8_t SFRPAGE_save = SFRPAGE;
   SFRPAGE = CONFIG_PAGE;

   P0SKIP   = 0xCF;                    // Skip P0.0-P0.3 and P0.6-P0.7

   P0MDOUT |= 0x10;                    // Enable UART0 TX as push-pull output

   #if LIN_TX_PUSH_PULL == 1
      P1MDOUT |= 0x09;                 // P1.0 (LIN0 TX) & P1.3 (LED) are push-pull
   #else   
      P1MDOUT |= 0x08;                 // P1.3 (LED) is push-pull
                                       // P1.0 (LIN0 TX) is open-drain      
   #endif

   XBR0     = 0x03;                    // Enable UART0
   XBR2     = 0x01;                    // Enable LIN0
   XBR2    |= 0x40;                    // Enable crossbar and weak pull-ups

   SFRPAGE = SFRPAGE_save;
}

//-----------------------------------------------------------------------------
// Check_Pin_State
//-----------------------------------------------------------------------------
//
// Return Value : None
// Parameters   : None
//
//
//-----------------------------------------------------------------------------
uint8_t Check_Pin_State (void)
{
   uint8_t SFRPAGE_save = SFRPAGE;
   SFRPAGE = CONFIG_PAGE;

   if (P1_4_SWITCH == 0)
   {
      return (PIN_ASSERTED); // Pin goes low when switch is pressed
   }
   else
   {
      return (PIN_NOT_ASSERTED);
   }

   SFRPAGE = SFRPAGE_save;
}

//-----------------------------------------------------------------------------
// Set_LED_State
//-----------------------------------------------------------------------------
//
// Return Value:  None
// Parameters:    None
//
//
//
//-----------------------------------------------------------------------------
void Set_LED_State (uint8_t state)
{
   uint8_t SFRPAGE_save = SFRPAGE;
   SFRPAGE = CONFIG_PAGE;

   switch (state)
   {
      case LED_OFF:
         P1_3_LED = 0;
         break;
      case LED_ON:
         P1_3_LED = 1;
         break;
      case LED_TOGGLE:
         P1_3_LED = !P1_3_LED;
         break;
      default:
         break;
   }

   SFRPAGE = SFRPAGE_save;
}

//-----------------------------------------------------------------------------
// End Of File
//-----------------------------------------------------------------------------
