//-----------------------------------------------------------------------------
// F50x_TargetApp_LIN0_BLsupport.c
//-----------------------------------------------------------------------------
// Copyright (C) 2010 Silicon Laboratories, Inc.
// http://www.silabs.com
//
// Program Description:
//
// [*** TEMPLATE INFORMATION ***]
// Filename: Fnnn_TargetApp_CommType_BLsupport.c
// nnn => Fill-in with the MCU family part number.
// CommType => Fill-in Comm interface name.
// Examples: F50x_TargetApp_CAN0_BLsupport.c, F34x_TargetApp_UART0_BLsupport.c
//
// This should typically be added to the application project and 
// included in that build.
// 
// Target Application FW can omit this file from build if 
// BL request via comm interface is not required or if it is implementing this
// functionality using a different mechanism.
//
// Target:         C8051Fxxx (Any Silicon Labs Flash MCU)
// Tool chain:     Generic
// Command Line:   None
//
//
// Release 1.0 / 01SEP2010 (PKC)
//    -Initial Revision
//
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
// Includes
//-----------------------------------------------------------------------------
#include <si_toolchain.h>
#include "C8051F500_defs.h"               // TODO: Change to actual file name

#include "Fxxx_Target_Config.h"
#include "Fxxx_Target_Interface.h"

#include "Fxxx_TargetApp_Config.h"

#define _INCLUDE_TARGET_ONLY_ITEMS_
#define _INCLUDE_TARGET_APP_ONLY_ITEMS_
#include "Fxxx_BL130_LIN0_Interface.h"  // TODO: Change to actual file name - DONE

//-----------------------------------------------------------------------------
// Global CONSTANTS
//-----------------------------------------------------------------------------

// Reserved space first byte
// *** Device-Specific ***
// TODO: Change this address to point to the first byte in reserved space
#define RESERVED_SPACE_ADDR (63*1024) // 0xFC00


#define READ_LIN0_REG(addr, target) { LIN0ADR = addr; target = LIN0DAT; }
#define WRITE_LIN0_REG(addr1, data1) { LIN0ADR = addr1; LIN0DAT = data1; }

//-----------------------------------------------------------------------------
// Global Variables
//-----------------------------------------------------------------------------


//-----------------------------------------------------------------------------
// Function Prototypes (Local)
//-----------------------------------------------------------------------------


//=============================================================================
// Function Definitions
//=============================================================================

//-----------------------------------------------------------------------------
// Comm_Init_AddOn
//-----------------------------------------------------------------------------
//
// Return Value : None
// Parameters   : None
//
//
//-----------------------------------------------------------------------------
/*
void Comm_Init_BL_Support_AddOn (void)
{
   // *** Comm-Specific / Device-Specific ***
   // TODO: Add code here to initialize the Comm Interface
   // Assume that this will be called from the application's own Comm_Init
   // function where all of the standard comm init will take place.
 
   // SFR page store/restore is not done here because this
   // will be called only from within the CAN0_Init of the Target App FW

}
*/

//-----------------------------------------------------------------------------
// Enter_BL_Mode
//-----------------------------------------------------------------------------
//
// Return Value : None
// Parameters   : None
//
// This function will cause a Flash Error Reset to enter BL mode.
//
//-----------------------------------------------------------------------------
uint8_t Enter_BL_Mode (void)
{
   // *** Device-Specific *** Change this as necessary based on MCU.
   // Note: Code-banked devices need additional steps.

   // The return value is just to prevent the compiler from optimizing out
   // the code read

   SI_VARIABLE_SEGMENT_POINTER(codeptr, uint8_t, SI_SEG_CODE);
   codeptr = (uint8_t SI_SEG_CODE *)(RESERVED_SPACE_ADDR);
   
   EA = 0; // Disable global interrupts
   
   return *codeptr; // This will cause a Flash Error Reset
}

//-----------------------------------------------------------------------------
// LIN0_Check_BL_Command
//-----------------------------------------------------------------------------
//
// Return Value : None
// Parameters   : None
//
// Checks if an BL command was received with matching serial number.
// Call this from the LIN0 ISR if data bytes were received for the 
// ID_TARGET_RX_BL_CMD LIN ID. If a match occurs, it will return a 1.
// Then the App FW can perform any necessary shutdown procedures and then call
// the Enter_BL_Mode function to reset the MCU and enter BL mode.
//
//-----------------------------------------------------------------------------
uint8_t LIN0_Check_BL_Command (void)
{
   // *** Comm-Specific / Device-Specific ***
   // TODO: Add code here as necessary for providing comm-specific support
   
   // SFR page store/restore is not done here because this
   // will be called only from within the CAN0_ISR

   uint8_t rx_data;
   uint8_t retval = 0;
   SI_VARIABLE_SEGMENT_POINTER(codeptr, uint8_t, SI_SEG_CODE);

   READ_LIN0_REG(LIN0DT1, rx_data); // Read the 1st data byte (command code)
   if (rx_data == TGT_CMD_ENTER_BL_MODE)
   {
      // Ignore the 2nd data byte (LIN0DT2) because that byte indicates if
      // a response is requested, but that is only meant for the BL FW.
      
      READ_LIN0_REG(LIN0DT3, rx_data); // Read the 3rd data byte (LIN serial0)
      codeptr = (uint8_t SI_SEG_CODE *)(APP_FW_INFOBLOCK_ADDR-APPIBO_TGT_DEVICE_SERIAL0);
      if (rx_data == *codeptr)
      {
         READ_LIN0_REG(LIN0DT4, rx_data); // Read the 4th data byte (LIN serial1)
         codeptr = (uint8_t SI_SEG_CODE *)(APP_FW_INFOBLOCK_ADDR-APPIBO_TGT_DEVICE_SERIAL1);
         if (rx_data == *codeptr)
         {  
            // If matched, then enter BL mode (no need to send a response to master)
            retval = 1;
         }
      }
   }

   return retval;
}

//-----------------------------------------------------------------------------
// End Of File
//-----------------------------------------------------------------------------
